/*
 *   Copyright 1996, University Corporation for Atmospheric Research.
 *   See ../COPYRIGHT file for copying and redistribution conditions.
 */
/* $Id: recs.c,v 1.1 1996/01/05 17:45:54 russ Exp $ */

#include <netcdf.h>
#include "ulog.h"
#include "timeunits.h"
#include "nc.h"
#include "recs.h"

/*
 * Initializes (reftime,valtime) pair table from open netcdf file.
 * Returns -1 on failure.
 */
int
new_recs(nc)
    ncfile *nc;
{
    int ncid = nc->ncid;
    int recid = nc->recid;
    long nrecs;
    long size;
    char *ncname = nc->ncname;

    /* get number of records */
    if (ncdiminq(ncid, recid, (char *)0, &nrecs) == -1) {
	uerror("%s: can't get number of records", ncname);
	return -1;
    }
    nc->rt = (rectimes *) emalloc(sizeof(rectimes));
    nc->rt->nrecs = nrecs;

    if (nrecs == 0) {
#define RECS_INIT_SIZE 32	/* initial size of growable record table */
	size = RECS_INIT_SIZE;
    } else {
	size = 2*nrecs;
    }
    nc->rt->size = size;
    nc->rt->reftimes = (double *) emalloc(size * sizeof(double));
    nc->rt->valtimes = (double *) emalloc(size * sizeof(double));
    
    if (nrecs > 0) {
	int nerrs = 0;
	/* int irec; */
	long start[1];
	long count[1];
	start[0] = 0;
	count[0] = nrecs;
	if (ncvarget(ncid, nc->reftimeid, start, count,
		     (void *)nc->rt->reftimes) == -1) {
	    uerror("%s: can't get reftimes", nc->ncname);
	    nerrs++;
	}
	if (ncvarget(ncid, nc->valtimeid, start, count,
		     (void *)nc->rt->valtimes ) == -1) {
	    uerror("%s: can't get valtimes", nc->ncname);
	    nerrs++;
	}
	if(nerrs)
	    return -1;
    }
    return 0;
}


/*
 * Frees memory used by (reftime,valtime) pair table from open netcdf file.
 */
int
free_recs(rp)
    rectimes *rp;
{
    if(rp) {
	if(rp->reftimes)
	    free(rp->reftimes);
	if(rp->valtimes)
	    free(rp->valtimes);
	free(rp);
    }
    return 0;
}


/*
 * Returns record number determined by (reftime,valtime) pair.  If
 * (reftime,valtime) is in table, returns corresponding record.  If
 * (reftime,valtime) is not in table, increments record count, enters
 * (reftime,valtime) for new record in table, updates reftime, valtime in
 * netCDF file, updates human-comprehensible time variables in netCDF file
 * (if any), and returns record.  Returns -1 on failure.
 */
long
getrec(nc, reftime, valtime, htp)
    ncfile *nc;
    double reftime;
    double valtime;
    humtime *htp;
{
  /* double *dp; */
    double *reftimes = nc->rt->reftimes;
    double *valtimes = nc->rt->valtimes;
    int ncid = nc->ncid;
    int i;

    /* First look in table of existing records */
    for (i=0; i < nc->rt->nrecs; i++) {
	if (reftime == reftimes[i] && valtime == valtimes[i])
	    return i;
    }

    /* Didn't find it, so create a new record */
    if (nc->rt->nrecs+1 == nc->rt->size) {/* no room for another record,
					     double size of table before
					     adding */
	nc->rt->size *= 2;
	nc->rt->reftimes = (double *) erealloc(nc->rt->reftimes,
		     nc->rt->size * sizeof(double));
	nc->rt->valtimes = (double *) erealloc(nc->rt->valtimes,
		     nc->rt->size * sizeof(double));
	reftimes = nc->rt->reftimes;
	valtimes = nc->rt->valtimes;
    }
    reftimes[nc->rt->nrecs] = reftime;
    valtimes[nc->rt->nrecs] = valtime;
				
    {				/* Update reftimes and valtimes in ncfile */
	long ix[2];
	long count[2];

	ix[0] = nc->rt->nrecs;
	count[0] = 1;

	if (ncvarput1(ncid, nc->reftimeid, ix, &reftime) == -1 ||
	    ncvarput1(ncid, nc->valtimeid, ix, &valtime) == -1) {
	    uerror("%s: failed to add new reftime, valtime", nc->ncname);
	    return -1;
	}
	/* update humtime record variables, if any, from htp */
	ix[1] = 0;
	if (nc->datetimeid > 0) {
	    char datetime[100];
	    sprintf(datetime, "%.4d-%.2d-%.2d %.2d:%.2d:%.2dZ",
		    htp->year, htp->month, htp->day,
		    htp->hour, htp->minute, htp->second);
	    count[1] = strlen(datetime)+1; /* include terminating null */
	    ncvarput(ncid, nc->datetimeid, ix, count, datetime);
	}
	if (nc->valoffsetid > 0) {
	    ncvarput1(ncid, nc->valoffsetid, ix, &htp->valoffset);
	}
    }
    nc->rt->nrecs++;
    return nc->rt->nrecs-1;
}
